/*
   urlencode.c
    1  2006-02-10  Matlib
    2  2009-02-23  Matlib  + link against libarguments
                           + add -r and -R options
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include <arguments.h>


#define BUFFER_SIZE 128

#define CHECK_ERROR(__COND__,__FUNC__) \
  { \
    if (__COND__) \
    { \
      perror (__FUNC__); \
      exit (EXIT_FAILURE); \
    } \
  }

unsigned char *enc_table [256];
unsigned char char_table [2 * 256];
unsigned char repl_table [4 * 256];


ARGUMENT_CALLBACK (argument_replace)
{
  int i;
  for (i = 0; parameter [i] != 0; i ++)
    enc_table [(unsigned char) parameter [i]] = 
       repl_table + 4 * (unsigned char) parameter [i];
}

ARGUMENT_CALLBACK (argument_no_replace)
{
  int i;
  for (i = 0; parameter [i] != 0; i ++)
    enc_table [(unsigned char) parameter [i]] =
       char_table + 2 * (unsigned char) parameter [i];
}

ARGUMENT_CALLBACK (argument_space)
{
  enc_table [' '] = (unsigned char *) "+";
}

struct Argument arguments [] =
{
  { 'r', "replace", argument_replace, ARGUMENT_F_PAR, NULL },
  { 'R', "no-replace", argument_no_replace, ARGUMENT_F_PAR, NULL },
  { 's', "space", argument_space, ARGUMENT_F_ONCE, NULL },
  { }
};


unsigned char digits [] = {'0', '1', '2', '3', '4', '5', '6', '7', 
                           '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

int main (int argc, char **argv)
{
  int i;
  unsigned int c;
  unsigned char buffer [BUFFER_SIZE];
  
  i = 0;
  for (c = 0; c < 256; c ++)
  {
    enc_table [c] = repl_table + i;
    repl_table [i ++] = '%';
    repl_table [i ++] = digits [c >> 4];
    repl_table [i ++] = digits [c & 15];
    repl_table [i ++] = 0;
  }
  i = 0;
  for (c = 0; c < 256; c ++)
  {
    if (((c >= 'A') && (c <= 'Z')) || ((c >= 'a') && (c <= 'z')) ||
       ((c >= '0') && (c <= '9')) || (c == '-') || (c == '_') ||
       (c == '.') || (c == '~'))
      enc_table [c] = char_table + i;
    char_table [i ++] = c;
    char_table [i ++] = 0;
  }
  
  i = arguments_parse ("urlencode", "2 (2009-02-23)", "[text]...", 
     "Converts plain text to URL encoded string.", argc, argv, arguments);
  
  if (i == argc)
  {
loop1:
    i = read (STDIN_FILENO, buffer, sizeof (buffer));
    CHECK_ERROR (i == -1, "read");
    if (i == 0)
      return EXIT_SUCCESS;
    for (c = 0; c < i; c ++)
      CHECK_ERROR (fputs ((char *) enc_table [buffer [c]], stdout) 
         == EOF, "fputs");
    goto loop1;
  }
  
loop2:
  for (c = 0; argv [i][c] != 0; c ++)
    CHECK_ERROR (fputs ((char *) enc_table [(unsigned char) argv [i][c]],
       stdout) == EOF, "fputs");
  i ++;
  if (i == argc)
    return EXIT_SUCCESS;
  CHECK_ERROR (fputs ((char *) enc_table [' '], stdout) == EOF, "fputs");
  goto loop2;
}
